/************************************************************************************

Copyright (c) 2001-2007  University of Washington Extension.

Module Name:

    tasks.c

Module Description:

    The tasks that are executed by the test application.

************************************************************************************/

#include "includes.h"
#include "print.h"
#include "uarts.h"
#include "init.h"

#define BUFSIZE 256

/************************************************************************************

   Allocate the stacks for each task.
   The maximum of tasks the application can have is defined by OS_MAX_TASKS in os_cfg.h

************************************************************************************/

static OS_STK   Task1Stk[APP_TASK_DEFAULT_STK_SIZE];
static OS_STK   Task2Stk[APP_TASK_DEFAULT_STK_SIZE];
static OS_STK   Task3Stk[APP_TASK_DEFAULT_STK_SIZE];


// Task prototypes
void TaskOne(void* pdata);
void TaskTwo(void* pdata);
void TaskThree(void* pdata);


// Useful functions
static void SetLED(BOOLEAN On);
void printWithBuf(char *buf, int size, char *format, ...);


/************************************************************************************

   This task is the initial task running, started by main(). It starts
   the system tick timer and creates all the other tasks. Then it deletes itself.

************************************************************************************/
void StartupTask(void* pdata)
{
	char buf[BUFSIZE];
	printWithBuf(buf, BUFSIZE, "StartupTask: Begin\n");
	printWithBuf(buf, BUFSIZE, "StartupTask: Starting timer tick\n");

    // Start the system tick
    Tmr_TickInit();

    // Create the the test tasks
    printWithBuf(buf, BUFSIZE, "StartupTask: Creating the 3 tasks\n");

    // The maximum of tasks the application can have is defined by OS_MAX_TASKS in os_cfg.h
    OSTaskCreate(TaskOne, (void*)0, (void*)&Task1Stk[APP_TASK_DEFAULT_STK_SIZE-1], APP_TASK_TEST1_PRIO);
    OSTaskCreate(TaskTwo, (void*)0, (void*)&Task2Stk[APP_TASK_DEFAULT_STK_SIZE-1], APP_TASK_TEST2_PRIO);
    OSTaskCreate(TaskThree, (void*)0, (void*)&Task3Stk[APP_TASK_DEFAULT_STK_SIZE-1], APP_TASK_TEST3_PRIO);

    // Delete ourselves, letting the work be done in the new tasks.
    printWithBuf(buf, BUFSIZE, "StartupTask: deleting self\n");
	OSTaskDel(OS_PRIO_SELF);
}

/************************************************************************************

   TaskOne turns on the LED every 1000 ticks.

************************************************************************************/
void TaskOne(void* pdata)
{
	char buf[BUFSIZE];
	printWithBuf(buf, BUFSIZE, "TaskOne: staring\n");

    while(1)
    {
    	OSTimeDly(1000);
    	printWithBuf(buf, BUFSIZE, "TaskOne: Turn LED On\n");
        SetLED(TRUE);
    }
}

/************************************************************************************

   TaskTwo turns off the LED every 500 ticks.

************************************************************************************/
void TaskTwo(void* pdata)
{
	char buf[BUFSIZE];
	printWithBuf(buf, BUFSIZE, "TaskTwo: staring\n");

    OSTimeDly(500);  // Delay starting to interleave the tasks.

    while(1)
    {
    	OSTimeDly(1000);
    	printWithBuf(buf, BUFSIZE, "TaskTwo: Turn LED OFF\n");
        SetLED(FALSE);
    }
}

/************************************************************************************

   TaskThree - Counts up

************************************************************************************/
void TaskThree(void* pdata)
{
	char buf[BUFSIZE];
	printWithBuf(buf, BUFSIZE, "TaskThree: staring\n");

	int count = 0;
    while(1)
    {
    	printWithBuf(buf, BUFSIZE, "TaskThree: count = %d\n", count++);
    	int i;
    	for(i=0;i<100000;i++); // pretend work
    }
}

/************************************************************************************

   Toggle the LED next to the display (labeled SD)

************************************************************************************/
static void SetLED(BOOLEAN On)
{
    if (On) {
        WRITEREG32(FIO0SET, SD_LED_BIT);
      } else {
        WRITEREG32(FIO0CLR, SD_LED_BIT);
     }
}

/************************************************************************************

   Print a formated string with the given buffer.
   Each task should use its own buffer to prevent data corruption.

************************************************************************************/
void printWithBuf(char *buf, int size, char *format, ...)
{
    va_list args;
    va_start(args, format);
    vsnprintf(buf, size, format, args);
	printString(buf);
    va_end(args);
}

